# Quartz Energy Mud Log OCR Show Reporting
> Deployment and Verification Guide

## Setup
- Ubuntu 16.04 (Tested on Amazon EC2, [https://aws.amazon.com/](https://aws.amazon.com/))
- MySQL 5.7
- tesseract-ocr
- Python 3.6 (Anaconda)

### MySQL

	$ sudo apt update
	$ sudo apt install -y mysql-server mysql-client
	... output omitted ...

	# test mysqladmin
	$ sudo mysqladmin -u root -h localhost --password=root_password
	... output omitted ...
	# press Control+D (or cmd-d? for IOS)

	# grant privileges to root user
	$ mysql -u root --password=root123
	... output omitted ...
	mysql> GRANT ALL PRIVILEGES ON *.* TO 'root'@'your_host_name' IDENTIFIED BY "password";
	Query OK, 0 rows affected (0.00 sec)

	mysql> FLUSH PRIVILEGES;
	Query OK, 0 rows affected (0.00 sec)
	
	mysql> exit
	Bye
	$ 


### tesseract-ocr

	$ sudo apt update
	$ sudo apt install -y g++
	$ sudo apt install -y tesseract-ocr tesseract-ocr-eng

	# tesseract and libleptonica dev
	$ sudo apt install libtesseract-dev libleptonica-dev

	# test it is installed
	$ tesseract --version

### Python 3.6 (Anaconda)

	# download Anaconda
	$ wget https://repo.continuum.io/archive/Anaconda3-4.4.0-Linux-x86_64.sh

	# launch installer
	$ bash Anaconda3-4.4.0-Linux-x86_64.sh
	# accept license terms, and defaults
	# allow adding directory to system path (recommended)
	# close terminal so path is refreshed next time

	# open terminal
	# create mudlog Python3.6 environment
	$ conda create --name mudlog python=3.6 anaconda  

	# activate environment
	$ source activate mudlog

	# install Python packages
	$ conda install pymysql
	$ conda install -c conda-forge python-dotenv=0.6.1
	$ conda install libgcc 
	$ sudo apt install -y imagemagick
	$ sudo apt install -y python-qt4


## Running

### SFTP to remote instance

Use an SFTP client (if you're on Windows, you can use Filezilla). Basically you add your private key and connect using SSH protocol.

Under Edit -> Settings Open the SFTP option and browse to the private key to add it.

Then add server File -> Site Manager. Define a name to identify connection. Important here is entering host name (your EC2 instance DNS). Specify you're using **SFTP protocol** and **ubuntu** as user name (no quotes).

Create **mud_log_ocr_v2** under **/home/ubuntu**: `/home/ubuntu/mud_log_ocr_v2` and transfer app code. This directory name is used for demo but you can use any directory of your choice. 

Transfer mud log files to a directory of your choice. 

### Edit environment variables first

	$ vi .env
	# ... example config variables are:
	# DB_HOST="localhost"
	# DB_USERNAME="username"
	# DB_PASSWORD="password"
	# DB_DATABASENAME="databasename"
	# SCORE_THERESHOLD=0 
	# 	^
	#   |
	#   `--- configurable threshold scoring parameter to display 
	#        all files which have a score above that value (report)

    # scores is a json with type, score, phrases and if matching phrases
    # are removed from ocr text after the type has been processed

	# Score definition (file already provided as part of submission)
    # vi scores.json
    # 
    # [
    #   { "type": "Show", 
    #     "score": 3, 
    #     "remove": false, 
    #     "phrases": ["oil show", "show", "o show", "oil on pits", "oil in samples", "flaring gas", "shown" ]
    #   },
    #   { "type": "Negative", 
    #     "score": 0, 
    #     "remove": true
    #     "phrases": ["no oil show", "no show", "no o show", "no oil on pits", ... ]
    #   },
    #   ...
    # ]

### Run OCR Extraction

	$ python extracttext.py <input_images_dir> <output_images_dir>

	# where:
	# 	input_images_dir: 	input images to process by OCR
	#   output_images_dir: 	output highlight and chart images here (WARNING: existing contents are removed when app starts)
	# for example:
	# python extracttext.py ../mud_images ../mud_highlights
	# ... at the end of the process the resulting extraction result report will be shown
	# and a chart will be saved as chart-report.png

### Run report

	# Second form usage: run report (please run ocr extraction first)
	$ python extracttext.py report <output_images_dir>
	# for example:
	# python extracttext.py report ../mud_highlights
	# where:
	#   output_images_dir: 	output highlight and chart images here

### Notes
	# If you get a warning saying font family cannot be found, try the following:
	$ sudo apt install font-manager
	$ sudo apt install msttcorefonts -qq
	$ rm ~/.cache/matplotlib -fr

### Verifying highlighting

While the app is running (or at the end if you prefer), you could download highlight images from the specified directory. Here's one of the highlighting images showing rectangles in red for words found.

<img src="doc/Figure_2.png">

## Sample Report

### Text Report

<pre>
=====================================
Image name: 43007102160000_sample2.tif
Phrase count    Score   Phrase (type)
------------    -----   ------------------------------
           2        6   << show >> ( Show )
           1        2   << oil stain >> ( Stain )
           2        4   << odor >> ( Stain )
           2        4   << cut >> ( Stain )
           2        4   << stain >> ( Stain )
Total Score for image name: 43007102160000_sample2.tif
Phrase count    Score   
------------    -----   
           9       20

(more image results here...)

Chart saved to ../mud_highlights/chart-report.png

</pre>

Some notes about the report:

- It shows the image name as a group header
- All phrases found are shown (`Phrase count`, `Score`, `Phrase` and its `type`)
- A total for is generated for all phrases in image
- At the end of the report `Chart saved to` indicates where the `Image by score` chart is saved.

### Chart

This shows a sample `Image by score` chart.

IMPORTANT: As the app is run in AWS infrastructure, you might need to download to desktop system to actually view the chart.

<img src="doc/Figure_1.png">

## Methodology

- As there are large files (in pixel height) split original image vertically
- OCR process use the sub-images created in the previous step. It also sets these flags at run-time that help reducing the number of false positives as seen in the `43047340800000_sample.tif` image that decrease the distance to ground truth from 83 to 33:
	- **language\_model\_penalty\_non\_freq\_dict\_word**	0.2     Penalty for words not in the frequent word dictionary
	- **language\_model\_penalty\_non\_dict\_word**	    0.3	Penalty for non-dictionary words
	- **--user-words** Use user-defined list of words. I put the words we're searching so the penalty for non-dictionary words is calculated by tesseract.
- For every sub-image run the OCR and add to raw text. Also generate a hocr file that will be used for highlighting matched words on the sub-image
- Each phrase type is highlighted with a separate color. The colors can be found in scores.json file
- When all sub-images were processed, save raw text and image name
- The saving function counts the "Show" occurrences inside raw text
- Save score by phrase (see `scores.json` for definition)
- Save score by image
- Run report (text and chart)
- Chart is saved in specified

## Data sources

The mud log images files (around 100) can be downloaded here:

[https://drive.google.com/file/d/0ByjxTGykXQjASVl1MnBPRldvSGs/view?usp=sharing](https://drive.google.com/file/d/0ByjxTGykXQjASVl1MnBPRldvSGs/view?usp=sharing) 

## Validation

Google sheets for validation results (Included in `Validation` sheet).

[https://docs.google.com/spreadsheets/d/1DH4dDqgXhnpfLaB8C933xykMwurQH-IUdV7c7CSqCqo/edit?usp=sharing](https://docs.google.com/spreadsheets/d/1DH4dDqgXhnpfLaB8C933xykMwurQH-IUdV7c7CSqCqo/edit?usp=sharing)

`tesseract 4 vs kraken` Shows a comparison between `kraken` ([https://pypi.python.org/pypi/kraken](https://pypi.python.org/pypi/kraken)) and `tesseract 4.0` alpha. As results were not good enough those libraries were discarded.  

## Future improvements

- Increase OCR matches maybe by removing grid lines and ability to recognize hand-written text (could take a lot of effort tuning train/test set)
